import 'dart:async';

import 'package:endless/stream/endless_stream_list_view.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:golden_toolkit/golden_toolkit.dart';

class MockItem {
  final String title;

  MockItem({required this.title});
}

void main() {
  testGoldens(
    'EndlessStreamListView',
    (WidgetTester tester) async {
      final streamController = StreamController<List<MockItem>>();

      final builder = GoldenBuilder.column()
        ..addScenario(
          'with items',
          EndlessStreamListView<MockItem>(
            loadMore: () {
              streamController.add([
                MockItem(title: 'Item 1'),
              ]);
            },
            stream: streamController.stream,
            itemBuilder: (
              context, {
              required item,
              required index,
              required totalItems,
            }) {
              return Container(
                color: Colors.purple,
                child: Text(item.title,
                    style: TextStyle(color: Colors.red, fontSize: 23)),
              );
            },
          ),
        );

      await tester.pumpWidgetBuilder(builder.build());
      await screenMatchesGolden(tester, 'endlessStreamListNotEmpty',
          customPump: (widget) {
        return widget.pump(Duration(seconds: 5));
      });
    },
  );

  testWidgets(
    'should not initially list items when loadOnSubscribe is false',
    (WidgetTester tester) async {
      final streamController = StreamController<List<MockItem>>();

      await tester.pumpWidget(
        MaterialApp(
          home: EndlessStreamListView<MockItem>(
            loadOnSubscribe: false,
            loadMore: () {
              streamController.add([MockItem(title: 'Item 1')]);
            },
            stream: streamController.stream,
            itemBuilder: (
              context, {
              required item,
              required index,
              required totalItems,
            }) {
              return Text(item.title);
            },
          ),
        ),
      );

      await tester.pump(Duration.zero);

      await expectLater(
        find.byType(MaterialApp),
        matchesGoldenFile('endlessStreamListViewEmpty.png'),
      );
    },
  );
}
